/* 
 * Copyright 2004 The Apache Software Foundation
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * Edited by Bjrn Harald Olsen 2006
 */
using System;
using Token = Lucene.Net.Analysis.Token;
using TokenFilter = Lucene.Net.Analysis.TokenFilter;
using TokenStream = Lucene.Net.Analysis.TokenStream;
namespace NorwegianAnalyzer
{
	
	/// <summary> A filter that stems norwegian words. It supports a table of words that should
	/// not be stemmed at all. The stemmer used can be changed at runtime after the
	/// filter object is created (as long as it is a NorwegianStemmer).
	/// 
	/// </summary>
	/// <author>     Bjrn Harald Olsen
	/// </author>
	/// <version>    $Id: NorwegianStemFilter.cs
	/// </version>	
	
	public sealed class NorwegianStemFilter : TokenFilter
	{
		/// <summary> The actual token in the input stream.</summary>
		private Token token = null;
		private NorwegianStemmer stemmer = null;		
		
		public NorwegianStemFilter(TokenStream in_Renamed) : base(in_Renamed)
		{
			stemmer = new NorwegianStemmer();
		}		
		
		/// <returns>  Returns the next token in the stream, or null at EOS
		/// </returns>
		public override Token Next()
		{
			if ((token = input.Next()) == null)
			{
				return null;
			}			
			else
			{					
				String s = stemmer.Stem(token.TermText());				
				// If not stemmed, dont waste the time creating a new token
				if (!s.Equals(token.TermText()))
				{
					return new Token(s, token.StartOffset(), token.EndOffset(), token.Type());
				}				
				return token;				
			}
		}
		
		/// <summary> Set a alternative/custom GermanStemmer for this filter.</summary>
		public void  SetStemmer(NorwegianStemmer stemmer)
		{
			if (stemmer != null)
			{
				this.stemmer = stemmer;
			}
		}
	}
}