/*
 * Copyright 2004 The Apache Software Foundation
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * Edited by Bjrn Harald Olsen 2006
 */
using System;
using Analyzer = Lucene.Net.Analysis.Analyzer;
using SimpleAnalyzer = Lucene.Net.Analysis.SimpleAnalyzer;
using FileDocument = NorwegianAnalyzer.FileDocument;
using IndexWriter = Lucene.Net.Index.IndexWriter;

namespace TrelsEval
{	
	class IndexTest
	{
        private static string[] NorwegianStopwords = new string[172] {"og", "i", "jeg", "det", "at", "en", "et", "den", "til", "er", "som", "p", "de",
"med", "han", "av", "ikke", "ikkje", "der", "s", "var", "meg", "seg", "men", "ett", "har", "om", "vi", "min", "mitt", "ha", "hadde", "hun", "n",
"over", "da", "ved", "fra", "du", "ut", "sin", "dem", "oss", "opp", "man", "kan", "hans", "hvor", "eller", "hva", "skal", "selv", "sjl", "her",
"alle", "vil", "bli", "ble", "blei", "blitt", "kunne", "inn", "nr", "vre", "kom", "noen", "noe", "ville", "dere", "deres", "kun", "ja", "etter", "ned",
"skulle", "denne", "for", "deg", "si", "sine", "sitt", "mot", "", "meget", "hvorfor", "dette", "disse", "uten", "hvordan", "ingen", "din", "ditt",
"blir", "samme", "hvilken", "hvilke", "snn", "inni", "mellom", "vr", "hver", "hvem", "hvis", "bde", "bare", "enn", "fordi", "fr", "mange",
"ogs", "slik", "vrt", "be", "begge", "siden", "dykk", "dykkar", "dei", "deira", "deires", "di", "d", "eg", "ein", "eit", "eitt", "elles", "honom",
"hj", "ho", "henne", "hennar", "hennes", "hoss", "hossen", "ingi", "inkje", "korleis", "korso", "kva", "kvar", "kvarhelst", "kven", "kvi", "kvifor",
"me", "medan", "mi", "mine", "mykje", "no", "nokon", "noka", "noko", "nokre", "sia", "sidan", "so", "somt", "somme", "upp", "vere", "vore",
"verte", "vort", "varte", "vart", "vors", "deim", "hoe", "nokor", "um"};

		[STAThread]
		public static void  Main(String[] args)
		{
			try
			{				
				DateTime start = DateTime.Now;

                // Norwegian document collection evaluation (requires the document collection)
                string docs = "C:\\Diplom\\AdressaD";
                string trels = "C:\\Diplom\\AdressaD_trels\\run1";
                string index = "C:\\Diplom\\AdressaD_index3";                          

                Analyzer analyzer = new NorwegianAnalyzer.NorwegianAnalyzer(NorwegianStopwords);                
                //new TrelsEval(docs, index, trels, analyzer, "norsk.txt");

                index = "C:\\Diplom\\AdressaD_indexSA";
                Analyzer sanalyzer = new Lucene.Net.Analysis.Standard.StandardAnalyzer();                
                //new TrelsEval(docs, index, trels, sanalyzer, "standard.txt");

                // Case 1 - output written to tscore.txt
                docs = "C:\\Diplom\\Oppdragelse\\docs";
                trels = "C:\\Diplom\\Oppdragelse\\trels";
                index = "C:\\Diplom\\Oppdragelse\\index";
                new TrelsCalc(docs, index, trels);

                // Case 2 - output written to tscore.txt
                docs = "C:\\Diplom\\Sjakkregler\\docs";
                trels = "C:\\Diplom\\Sjakkregler\\trels";
                index = "C:\\Diplom\\Sjakkregler\\index";
                new TrelsCalc(docs, index, trels);
                
				System.DateTime end = System.DateTime.Now;
				System.TimeSpan ts = end - start;
				System.Console.Out.Write(ts.TotalSeconds);
				System.Console.Out.WriteLine(" total milliseconds");
								
				System.Console.Out.WriteLine(" free memory before gc");
				System.Console.Out.Write(System.GC.GetTotalMemory(true));
				System.Console.Out.WriteLine(" total memory before gc");
				
				System.GC.Collect();				
                
                System.Console.Out.WriteLine(" free memory after gc");
                System.Console.Out.Write(System.GC.GetTotalMemory(true));
                System.Console.Out.WriteLine(" total memory after gc");				
			}
			catch (System.Exception e)
			{
				System.Console.Out.WriteLine(e.ToString());
			}				
			    System.Console.ReadLine();
		}
		
		public static void  IndexDocs(IndexWriter writer, System.IO.FileInfo file)
		{
			if (System.IO.Directory.Exists(file.FullName))
			{
				System.String[] files = System.IO.Directory.GetFileSystemEntries(file.FullName);
				foreach(string filename in files)
					IndexDocs(writer, new System.IO.FileInfo(filename));
			}
			else
			{
				System.Console.Out.WriteLine("adding " + file);
				writer.AddDocument(FileDocument.Document(file));
			}
		}
	}
}