/*
 * Author Bjrn Harald Olsen 2006
 */
using System;
using System.Collections;

namespace TrelsEval
{
	public class RelevanceSet
	{
		private string query;
		private ArrayList offTopics;
		private ArrayList onTopics;
        private int[] priorityList;
        private Random random;       

		public RelevanceSet(string query, ArrayList offTopics, ArrayList onTopics)
		{
			this.query=query;
			this.offTopics=offTopics;
			this.onTopics=onTopics;
            priorityList = null;
            random = new Random();
		}

		public RelevanceSet(string query)
		{
			this.query=query;            
			offTopics = new ArrayList();
			onTopics = new ArrayList();
            random = new Random();
		}

        public void setPriorityList(int[] p)
        {
            if (p.Length == onTopics.Count && p.Length == offTopics.Count)
            {
                priorityList = p;
            }
            else
            {
                Console.WriteLine("Priority list is either wrong length or there are different number of onTopics and offTopics. Priority list ignored.");
            }
        }

        public int numOnt()
        {
            return onTopics.Count;
        }

		public void addOffTopic(string offT)
		{
			offTopics.Add(offT);           
		}

		public void addOnTopic(string onT)
		{
			onTopics.Add(onT);            
		}

		public IList getOffTopics()
		{
			return offTopics;
		}

		public IList getOnTopics()
		{
			return onTopics;
		}

        public IList getOffTopics(int n)
        {
            ArrayList t = new ArrayList();
            if (priorityList != null)
            {
                for (int i = 0; i < n; ++i)
                    t.Add(offTopics[priorityList[i]]);
            }
            else
            {
                for (int i = 0; i < n; ++i)
                    t.Add(offTopics[i]);
            }
            return t;
        }

        public IList getOnTopics(int n)
        {
            ArrayList t = new ArrayList();
            if (priorityList != null)
            {
                for (int i = 0; i < n; ++i)
                    t.Add(onTopics[(int)priorityList[i]]);
            }
            else
            {
                for (int i = 0; i < n; ++i)
                    t.Add(onTopics[i]);
            }
            return t;
        }

        public IList getSubOnTopics(int num)
        {
            ArrayList all = (ArrayList)onTopics.Clone();
            ArrayList res = new ArrayList();
            int r = 0;
            for (int i = 0; i < Math.Min(num,onTopics.Count); ++i)
            {
                r = random.Next(all.Count);
                res.Add(all[r]);
                all.RemoveAt(r);
            }
            return res;
        }

        public IList getSubOffTopics(int num)
        {
            ArrayList all = (ArrayList)offTopics.Clone();
            ArrayList res = new ArrayList();
            int r = 0;
            for (int i = 0; i < Math.Min(num,offTopics.Count); ++i)
            {
                r = random.Next(all.Count);
                res.Add(all[r]);
                all.RemoveAt(r);
            }
            return res;
        }

		public string getQuery()
		{
			return query;
		}

		override public string ToString()
		{
			string result="TREL: "+query;
			string buf=" offTopics: ";
			foreach (string t in offTopics)
			{
				buf+=t+", ";
			}
			result+=buf;
			buf=" onTopics: ";
			foreach(string t in onTopics)
			{
				buf+=t+", ";
			}
			return result+buf;
		}
	}
}
