/*
 * Author Bjrn Harald Olsen 2006
 * 
 */
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Collections;
using System.Text;
using System.Text.RegularExpressions;

namespace TrelsEval
{
    class TrelsCalc
    {	
		private string pathIndex;
		private string pathDoc;
		
		private string pathTrels;				
		private ArrayList trelsSet;

		private double BETA = 1.0;
        
		public TrelsCalc(string pathDoc, string pathIndex, string pathTrels)
		{
			this.pathIndex=pathIndex;
			this.pathDoc=pathDoc;
		
			this.pathTrels=pathTrels;
			this.trelsSet=new ArrayList();
            
            Console.WriteLine("Loading Trels...");
            loadTrels(new DirectoryInfo(pathTrels));
            sortTrels();
            //calc();
            //calcRandomMean();
            calcPriorityList();                        
		}

        private void sortTrels()
        {
            RelevanceSet tmp=null;
            for (int i = 0; i < trelsSet.Count; i++)
            {
                for (int j = i+1; j < trelsSet.Count; j++)
                {
                    if (((RelevanceSet)trelsSet[j]).numOnt() < ((RelevanceSet)trelsSet[i]).numOnt())
                    {
                        tmp = (RelevanceSet)trelsSet[i];
                        trelsSet[i] = trelsSet[j];
                        trelsSet[j] = tmp;
                    }
                }
            }
        }

        private void loadTrels(DirectoryInfo trelsDir)
		{
			RelevanceSet buf;
			foreach (FileInfo fi in trelsDir.GetFiles("*.xml"))
			{
				// skip temporary office files
				if (fi.Name.StartsWith("~"))
					continue;

				try 
				{
					buf = TrelsXmlReader.Parse(fi);
					if(buf!=null) trelsSet.Add(buf);
				}
				catch (Exception e)
				{					
					Console.WriteLine(e.ToString());
				}
			}
		}

        private void calc()
        {
            DateTime start = DateTime.Now;           
            ArrayList scores = new ArrayList();
            ArrayList scoresAlt = new ArrayList();
            DirectoryInfo directory = new DirectoryInfo(pathDoc);
            FileInfo[] files = directory.GetFiles("*.txt");
            StreamWriter sw = new StreamWriter("tscore.txt");
            StreamWriter swAlt = new StreamWriter("tscoreAlt.txt");

            foreach (RelevanceSet trel in trelsSet)
            {
                //Console.WriteLine(trel.getQuery());
                //Console.WriteLine("heisann");
                string text = "";
                foreach (FileInfo fi in files)
                {
                    //Console.WriteLine("loading file " +fi.FullName);
                    // skip temporary office files
                    if (fi.Name.StartsWith("~"))
                        continue;
                    try
                    {                        
                        text = Parser.Parse(fi.FullName);
                    }
                    catch (Exception)
                    {
                        Console.WriteLine("could not pick file");
                        continue;
                    }

                    text=text.ToLower();

                    int numOn = 0;
                    int numOff = 0;
                    int numOnAlt = 0;
                    int numOffAlt = 0;

                    foreach (string ont in trel.getOnTopics())
                    {                        
                        MatchCollection matches = Regex.Matches(text, ont+"[,. ]");                     
                        numOn+=matches.Count;
                        Match m = Regex.Match(text, ont + "[,. ]");
                        if (m.Success)
                        {
                            //Console.WriteLine(m.Value);
                            numOnAlt++;
                        }
                    }
                    foreach (string off in trel.getOffTopics())
                    {
                        MatchCollection matches = Regex.Matches(text, off + "[,. ]");
                        numOff += matches.Count;
                        Match m = Regex.Match(text, off + "[,. ]");
                        if (m.Success)
                        {
                            //Console.WriteLine("off: "+m.Value);
                            numOffAlt++;
                        }
                    }
                    
                    MatchCollection docCount = Regex.Matches(text, "[a-,.] ");           
                    int docSize = docCount.Count/100;
                    //Console.WriteLine("docsize: " + docSize);
                    int numOnTopics = trel.numOnt();

                    double tScore_basic = (numOn - (BETA * numOff))/docSize*10;
                    double tScore_basicAlt = (numOnAlt - (BETA * numOffAlt))/docSize*10;

                    scoresAlt.Add(tScore_basicAlt);
                    scores.Add(tScore_basic);
                }

                sw.WriteLine(trel.numOnt() + " " + tScoreD3(scores) + " " + tScoreD4(scores));
                swAlt.WriteLine(trel.numOnt() + " " + tScoreD3(scoresAlt) + " " + tScoreD4(scoresAlt));
                
                scores.Clear();
                scoresAlt.Clear();
            }
            sw.Close();
            swAlt.Close();
        }

        private int[] reverse(int[] l)
        {
            int[] t = new int[l.Length];
            for (int i = 0; i < l.Length; i++)
                t[l.Length - i - 1] = l[i];
            return t;
        }

        private void calcPriorityList()
        {
            DateTime start = DateTime.Now;
            ArrayList scores = new ArrayList();
            ArrayList scoresAlt = new ArrayList();
            DirectoryInfo directory = new DirectoryInfo(pathDoc);
            FileInfo[] files = directory.GetFiles("*.txt");
            StreamWriter sw = new StreamWriter("tscore.txt");
            StreamWriter swAlt = new StreamWriter("tscoreAlt.txt");

            if (trelsSet.Count > 1)
            {
                Console.WriteLine("calcPriorityList() can only be used with one Trel. Use calc() instead.");
                return;
            }
            RelevanceSet trels = (RelevanceSet)trelsSet[0];
            int[] priorityList ={ 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19 };
            int[] priorityListBestFirst ={ 4, 6, 10, 19, 18, 0, 3, 2, 8, 16,12, 1, 9, 5, 17, 7, 11, 13, 14, 15 };
            int[] priorityListBestLast = reverse(priorityListBestFirst);
            trels.setPriorityList(priorityList);

            for (int i = 1; i <= trels.numOnt(); ++i )
            {
                //Console.WriteLine(trel.getQuery());                
                string text = "";
                int totOff = 0;
                int totOn = 0;
                foreach (FileInfo fi in files)
                {
                    //Console.WriteLine("loading file " +fi.FullName);
                    // skip temporary office files
                    if (fi.Name.StartsWith("~"))
                        continue;
                    try
                    {
                        text = Parser.Parse(fi.FullName);
                    }
                    catch (Exception)
                    {
                        Console.WriteLine("could not pick file");
                        continue;
                    }
                    if(i==1) Console.WriteLine(fi.FullName);

                    text = text.ToLower();

                    int numOn = 0;
                    int numOff = 0;
                    int numOnAlt = 0;
                    int numOffAlt = 0;

                    foreach (string ont in trels.getOnTopics(i))
                    {
                        MatchCollection matches = Regex.Matches(text, ont + "[,. ]");
                        numOn += matches.Count;
                        Match m = Regex.Match(text, ont + "[,. ]");
                        if (m.Success)
                        {
                            //Console.WriteLine(m.Value);
                            numOnAlt++;
                        }
                    }
                    foreach (string off in trels.getOffTopics(i))
                    {
                        MatchCollection matches = Regex.Matches(text, off + "[,. ]");
                        numOff += matches.Count;
                        Match m = Regex.Match(text, off + "[,. ]");
                        if (m.Success)
                        {
                            //Console.WriteLine("off: "+m.Value);
                            numOffAlt++;
                        }
                    }

                    MatchCollection docCount = Regex.Matches(text, "[a-,.] ");
                    int docSize = docCount.Count / 100;
                    //Console.WriteLine("docsize: " + docSize);
                    int numOnTopics = trels.numOnt();
                    totOff += numOff;
                    totOn += numOn;
                    double tScore_basic = (numOn - (BETA * numOff)) / docSize * 10;
                    double tScore_basicAlt = (numOnAlt - (BETA * numOffAlt)) / docSize * 10;

                    scoresAlt.Add(tScore_basicAlt);
                    scores.Add(tScore_basic);
                }

                //Console.WriteLine("offt: " + totOff + "(" + i + ")");
                //Console.WriteLine("on: " + totOn + "(" + i + ")");
                sw.WriteLine(i + " " + tScoreD3(scores) + " " + tScoreD4(scores));
                swAlt.WriteLine(i + " " + tScoreD3(scoresAlt) + " " + tScoreD4(scoresAlt));

                scores.Clear();
                scoresAlt.Clear();
            }
            sw.Close();
            swAlt.Close();
        }

        private void calcRandomMean()
        {
            DateTime start = DateTime.Now;
            ArrayList scores = new ArrayList();
            ArrayList scoresAlt = new ArrayList();
            DirectoryInfo directory = new DirectoryInfo(pathDoc);
            FileInfo[] files = directory.GetFiles("*.txt");
            StreamWriter sw = new StreamWriter("tscore.txt");
            StreamWriter swAlt = new StreamWriter("tscoreAlt.txt");
            
            string text = "";
            RelevanceSet trel = (RelevanceSet)trelsSet[trelsSet.Count - 1];
            Console.WriteLine("trel-lengde: " + trel.numOnt());

            for (int t = 1; t <= 20; t++)
            {
                double Dq3 = 0.0;
                double Dq4 = 0.0;
                double DqAlt3 = 0.0;
                double DqAlt4 = 0.0;

                for (int i = 0; i < 200; i++)
                {
                    foreach (FileInfo fi in files)
                    {
                        if (fi.Name.StartsWith("~"))
                            continue;
                        try
                        {
                            text = Parser.Parse(fi.FullName);
                        }
                        catch (Exception)
                        {
                            Console.WriteLine("could not pick file");
                            continue;
                        }

                        text = text.ToLower();

                        int numOn = 0;
                        int numOff = 0;
                        int numOnAlt = 0;
                        int numOffAlt = 0;

                        foreach (string ont in trel.getSubOnTopics(t))
                        {
                            MatchCollection matches = Regex.Matches(text, ont + "[,. ]");
                            numOn += matches.Count;
                            Match m = Regex.Match(text, ont + "[,. ]");
                            if (m.Success) numOnAlt++;                            
                        }
                        foreach (string off in trel.getSubOffTopics(t))
                        {
                            MatchCollection matches = Regex.Matches(text, off + "[,. ]");
                            numOff += matches.Count;
                            Match m = Regex.Match(text, off + "[,. ]");
                            if (m.Success) numOffAlt++;                            
                        }

                        MatchCollection docCount = Regex.Matches(text, "[a-,.] ");
                        int docSize = docCount.Count / 100;                        
                        int numOnTopics = trel.numOnt();

                        double tScore_basic = (numOn - (BETA * numOff)) / docSize * 10;
                        double tScore_basicAlt = (numOnAlt - (BETA * numOffAlt)) / docSize * 10;

                        scoresAlt.Add(tScore_basicAlt);
                        scores.Add(tScore_basic);
                    }

                    Dq3 += tScoreD3(scores);
                    Dq4 += tScoreD4(scores);
                    DqAlt3 += tScoreD3(scoresAlt);
                    DqAlt4 += tScoreD4(scoresAlt);

                    scores.Clear();
                    scoresAlt.Clear();
                }

                sw.WriteLine(t + " " + Dq3/200 + " " + Dq4/200);
                swAlt.WriteLine(t + " " + DqAlt3/200 + " " + DqAlt4/200);
            }                
            
            sw.Close();
            swAlt.Close();
        }

        private double tScoreD3(ArrayList scores)
        {
            double numerator = 0.0;
            double denominator = 0.0;
            for (int j = 1; j <= scores.Count; ++j)
            {
                numerator += (((double)scores[j - 1]) * (1.0 / (double)j));
                denominator += (1.0 / (double)j);
            }
            return numerator / denominator;
        }

        private double tScoreD4(ArrayList scores)
        {
            double tScoreTest = 0.0;
            foreach (double s in scores)
                tScoreTest += s;
            return tScoreTest / scores.Count;
        }
	}
}
