/*
 * NUnit test for RelevanceSet
 * 
 * Author Bjrn Harald Olsen 2006
 * 
 */
using System;
using System.Collections;
using System.Collections.Generic;
using NUnit.Framework;

namespace TrelsEval.NUnit
{
    [TestFixture]
    public class RelevanceSetTest
    {
        private RelevanceSet trel;

        [Test]
        public void CreateTrel()
        {
            string testString = "query";
            trel = new RelevanceSet(testString);
            Assert.AreEqual(testString, trel.getQuery());
        }

        [Test]
        public void AddTopics()
        {
            trel = new RelevanceSet("query");
            trel.addOffTopic("test1");
            trel.addOffTopic("test2");
            IList onT = trel.getOnTopics();
            IList offT = trel.getOffTopics();
            Assert.AreEqual(0, onT.Count);
            Assert.AreEqual(2, offT.Count);

            trel.addOnTopic("test3");
            trel.addOnTopic("test4");
            onT = trel.getOnTopics();
            offT = trel.getOffTopics();
            Assert.AreEqual(2, onT.Count);
            Assert.AreEqual(2, offT.Count);
        }

        [Test]
        public void RandomSubTopicSet()
        {
            trel = new RelevanceSet("query");
            trel.addOffTopic("test1");
            trel.addOffTopic("test2");            
            trel.addOnTopic("test3");
            trel.addOnTopic("test4");            
            IList onT;
            IList offT;
           
            onT = trel.getSubOnTopics(2);
            Assert.AreEqual(2, onT.Count);
            Assert.AreNotEqual((string)onT[0], (string)onT[1]);          
           
            offT = trel.getSubOffTopics(2);
            Assert.AreEqual(2, offT.Count);
            Assert.AreNotEqual((string)offT[0], (string)offT[1]);           
        }
    }
}
